@@ -16,9 +16,10 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <lib/webauth.h>
 #include <tests/tap/basic.h>
 #include <tests/tap/kerberos.h>
+#include <webauth.h>
+#include <webauth/basic.h>
 
 #define CHECK(s, c, m) check_status(s, c, m, __FILE__, __LINE__)
 
@@ -29,11 +30,11 @@ check_status(int s, WEBAUTH_KRB5_CTXT *c, const char *message,
 {
     if (s == WA_ERR_KRB5 && c != NULL)
         diag("webauth call failed %s line %lu: %s (%d): %s %d\n", file, line,
-             webauth_error_message(s), s, webauth_krb5_error_message(c),
+             webauth_error_message(NULL, s), s, webauth_krb5_error_message(c),
              webauth_krb5_error_code(c));
     else if (s != WA_ERR_NONE)
         diag("webauth call failed %s line %lu: %s (%d)\n", file, line,
-             webauth_error_message(s), s);
+             webauth_error_message(NULL, s), s);
     is_int(s, WA_ERR_NONE, "%s", message);
 }
 
@@ -136,7 +137,8 @@ do_import(const char *keytab)
 int 
 main(void)
 {
-    char *principal, *keytab;
+    const char *principal;
+    char *keytab;
 
     /* Read the configuration information. */
     principal = kerberos_setup();
@@ -149,6 +151,5 @@ main(void)
     do_export(principal, getenv("KRB5CCNAME"));
     do_import(keytab);
 
-    kerberos_cleanup();
     return 0;
 }
